/*
 * Jsonix is a JavaScript library which allows you to convert between XML
 * and JavaScript object structures.
 *
 * Copyright (c) 2010, Aleksei Valikov, Highsource.org
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Aleksei Valikov nor the
 *       names of contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ALEKSEI VALIKOV BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

Jsonix.Request = Jsonix.Class({
	factories : [ function() {
		return new XMLHttpRequest();
	}, function() {
		return new ActiveXObject('Msxml2.XMLHTTP');
	}, function() {
		return new ActiveXObject("Msxml2.XMLHTTP.6.0");
	}, function() {
		return new ActiveXObject("Msxml2.XMLHTTP.3.0");
	}, function() {
		return new ActiveXObject('Microsoft.XMLHTTP');
	}

	],
	initialize : function() {
	},
	issue : function(url, onSuccess, onFailure, options) {
		Jsonix.Util.Ensure.ensureString(url);
		if (Jsonix.Util.Type.exists(onSuccess)) {
			Jsonix.Util.Ensure.ensureFunction(onSuccess);
		} else {
			onSuccess = function() {
			};
		}
		if (Jsonix.Util.Type.exists(onFailure)) {
			Jsonix.Util.Ensure.ensureFunction(onFailure);
		} else {
			onFailure = function() {
			};
		}
		if (Jsonix.Util.Type.exists(options)) {
			Jsonix.Util.Ensure.ensureObject(options);
		} else {
			options = {};
		}

		var transport = this.createTransport();

		var method = Jsonix.Util.Type.isString(options.method) ? options.method : 'GET';
		var async = Jsonix.Util.Type.isBoolean(options.async) ? options.async : true;
		var proxy = Jsonix.Util.Type.isString(options.proxy) ? options.proxy : Jsonix.Request.PROXY;

		var user = Jsonix.Util.Type.isString(options.user) ? options.user : null;
		var password = Jsonix.Util.Type.isString(options.password) ? options.password : null;

		if (Jsonix.Util.Type.isString(proxy) && (url.indexOf("http") === 0)) {
			url = proxy + encodeURIComponent(url);
		}

		if (Jsonix.Util.Type.isString(user)) {
			transport.open(method, url, async, user, password);
		} else {
			transport.open(method, url, async);
		}

		if (Jsonix.Util.Type.isObject(options.headers)) {

			for ( var header in options.headers) {
				if (options.headers.hasOwnProperty(header)) {
					transport.setRequestHeader(header, options.headers[header]);
				}
			}
		}

		var data = Jsonix.Util.Type.exists(options.data) ? options.data : null;
		if (!async) {
			transport.send(data);
			this.handleTransport(transport, onSuccess, onFailure);
		} else {
			var that = this;

			transport.onreadystatechange = function() {
				that.handleTransport(transport, onSuccess, onFailure);
			};

			window.setTimeout(function() {
				transport.send(data);
			}, 0);
		}
		return transport;

	},
	handleTransport : function(transport, onSuccess, onFailure) {
		if (transport.readyState == 4) {
			if (!transport.status || (transport.status >= 200 && transport.status < 300)) {
				onSuccess(transport);
			}
			if (transport.status && (transport.status < 200 || transport.status >= 300)) {
				onFailure(transport);
			}
		}
	},
	createTransport : function() {
		for ( var index = 0, length = this.factories.length; index < length; index++) {
			try {
				var transport = this.factories[index]();
				return transport;
			} catch (e) {
				// TODO log
			}
		}
		throw 'Could not create XML HTTP transport.';
	},
	CLASS_NAME : 'Jsonix.Request'
});
Jsonix.Request.INSTANCE = new Jsonix.Request();
Jsonix.Request.PROXY = null;